import { Morphir_IR_FQName } from "./../../morphir/ir/FQName";
import { Morphir_IR_Literal } from "./../../morphir/ir/Literal";
import { Morphir_IR_Name } from "./../../morphir/ir/Name";
import { Morphir_IR_Type } from "./../../morphir/ir/Type";
export declare namespace Morphir_IR_Value {
    type Definition<Ta, Va> = {
        inputTypes: Array<[Morphir_IR_Name.Name, Va, Morphir_IR_Type.Type<Ta>]>;
        outputType: Morphir_IR_Type.Type<Ta>;
        body: Morphir_IR_Value.Value<Ta, Va>;
    };
    function decodeDefinition<Ta, Va>(decodeTa: (input: any) => Ta, decodeVa: (input: any) => Va, input: any): Definition<Ta, Va>;
    function encodeDefinition<Ta, Va>(encodeTa: (value: Ta) => any, encodeVa: (value: Va) => any, value: Definition<Ta, Va>): any;
    type Pattern<A> = AsPattern<A> | ConstructorPattern<A> | EmptyListPattern<A> | HeadTailPattern<A> | LiteralPattern<A> | TuplePattern<A> | UnitPattern<A> | WildcardPattern<A>;
    function decodePattern<A>(decodeA: (input: any) => A, input: any): Pattern<A>;
    function encodePattern<A>(encodeA: (value: A) => any, value: Pattern<A>): any;
    class AsPattern<A> {
        arg1: A;
        arg2: Morphir_IR_Value.Pattern<A>;
        arg3: Morphir_IR_Name.Name;
        kind: "AsPattern";
        constructor(arg1: A, arg2: Morphir_IR_Value.Pattern<A>, arg3: Morphir_IR_Name.Name);
    }
    function decodeAsPattern<A>(decodeA: (input: any) => A, input: any): AsPattern<A>;
    function encodeAsPattern<A>(encodeA: (value: A) => any, value: AsPattern<A>): any;
    class ConstructorPattern<A> {
        arg1: A;
        arg2: Morphir_IR_FQName.FQName;
        arg3: Array<Morphir_IR_Value.Pattern<A>>;
        kind: "ConstructorPattern";
        constructor(arg1: A, arg2: Morphir_IR_FQName.FQName, arg3: Array<Morphir_IR_Value.Pattern<A>>);
    }
    function decodeConstructorPattern<A>(decodeA: (input: any) => A, input: any): ConstructorPattern<A>;
    function encodeConstructorPattern<A>(encodeA: (value: A) => any, value: ConstructorPattern<A>): any;
    class EmptyListPattern<A> {
        arg1: A;
        kind: "EmptyListPattern";
        constructor(arg1: A);
    }
    function decodeEmptyListPattern<A>(decodeA: (input: any) => A, input: any): EmptyListPattern<A>;
    function encodeEmptyListPattern<A>(encodeA: (value: A) => any, value: EmptyListPattern<A>): any;
    class HeadTailPattern<A> {
        arg1: A;
        arg2: Morphir_IR_Value.Pattern<A>;
        arg3: Morphir_IR_Value.Pattern<A>;
        kind: "HeadTailPattern";
        constructor(arg1: A, arg2: Morphir_IR_Value.Pattern<A>, arg3: Morphir_IR_Value.Pattern<A>);
    }
    function decodeHeadTailPattern<A>(decodeA: (input: any) => A, input: any): HeadTailPattern<A>;
    function encodeHeadTailPattern<A>(encodeA: (value: A) => any, value: HeadTailPattern<A>): any;
    class LiteralPattern<A> {
        arg1: A;
        arg2: Morphir_IR_Literal.Literal;
        kind: "LiteralPattern";
        constructor(arg1: A, arg2: Morphir_IR_Literal.Literal);
    }
    function decodeLiteralPattern<A>(decodeA: (input: any) => A, input: any): LiteralPattern<A>;
    function encodeLiteralPattern<A>(encodeA: (value: A) => any, value: LiteralPattern<A>): any;
    class TuplePattern<A> {
        arg1: A;
        arg2: Array<Morphir_IR_Value.Pattern<A>>;
        kind: "TuplePattern";
        constructor(arg1: A, arg2: Array<Morphir_IR_Value.Pattern<A>>);
    }
    function decodeTuplePattern<A>(decodeA: (input: any) => A, input: any): TuplePattern<A>;
    function encodeTuplePattern<A>(encodeA: (value: A) => any, value: TuplePattern<A>): any;
    class UnitPattern<A> {
        arg1: A;
        kind: "UnitPattern";
        constructor(arg1: A);
    }
    function decodeUnitPattern<A>(decodeA: (input: any) => A, input: any): UnitPattern<A>;
    function encodeUnitPattern<A>(encodeA: (value: A) => any, value: UnitPattern<A>): any;
    class WildcardPattern<A> {
        arg1: A;
        kind: "WildcardPattern";
        constructor(arg1: A);
    }
    function decodeWildcardPattern<A>(decodeA: (input: any) => A, input: any): WildcardPattern<A>;
    function encodeWildcardPattern<A>(encodeA: (value: A) => any, value: WildcardPattern<A>): any;
    type RawValue = Morphir_IR_Value.Value<[], []>;
    function decodeRawValue(input: any): RawValue;
    function encodeRawValue(value: RawValue): any;
    type Specification<Ta> = {
        inputs: Array<[Morphir_IR_Name.Name, Morphir_IR_Type.Type<Ta>]>;
        output: Morphir_IR_Type.Type<Ta>;
    };
    function decodeSpecification<Ta>(decodeTa: (input: any) => Ta, input: any): Specification<Ta>;
    function encodeSpecification<Ta>(encodeTa: (value: Ta) => any, value: Specification<Ta>): any;
    type TypedValue = Morphir_IR_Value.Value<[], Morphir_IR_Type.Type<[]>>;
    function decodeTypedValue(input: any): TypedValue;
    function encodeTypedValue(value: TypedValue): any;
    type Value<Ta, Va> = Apply<Va, Ta> | Constructor<Va> | Destructure<Va, Ta> | Field<Va, Ta> | FieldFunction<Va> | IfThenElse<Va, Ta> | Lambda<Va, Ta> | LetDefinition<Va, Ta> | LetRecursion<Va, Ta> | List<Va, Ta> | Literal<Va> | PatternMatch<Va, Ta> | Record<Va, Ta> | Reference<Va> | Tuple<Va, Ta> | Unit<Va> | UpdateRecord<Va, Ta> | Variable<Va>;
    function decodeValue<Ta, Va>(decodeTa: (input: any) => Ta, decodeVa: (input: any) => Va, input: any): Value<Ta, Va>;
    function encodeValue<Ta, Va>(encodeTa: (value: Ta) => any, encodeVa: (value: Va) => any, value: Value<Ta, Va>): any;
    class Apply<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Value<Ta, Va>;
        arg3: Morphir_IR_Value.Value<Ta, Va>;
        kind: "Apply";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Value<Ta, Va>, arg3: Morphir_IR_Value.Value<Ta, Va>);
    }
    function decodeApply<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): Apply<Va, Ta>;
    function encodeApply<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: Apply<Va, Ta>): any;
    class Constructor<Va> {
        arg1: Va;
        arg2: Morphir_IR_FQName.FQName;
        kind: "Constructor";
        constructor(arg1: Va, arg2: Morphir_IR_FQName.FQName);
    }
    function decodeConstructor<Va>(decodeVa: (input: any) => Va, input: any): Constructor<Va>;
    function encodeConstructor<Va>(encodeVa: (value: Va) => any, value: Constructor<Va>): any;
    class Destructure<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Pattern<Va>;
        arg3: Morphir_IR_Value.Value<Ta, Va>;
        arg4: Morphir_IR_Value.Value<Ta, Va>;
        kind: "Destructure";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Pattern<Va>, arg3: Morphir_IR_Value.Value<Ta, Va>, arg4: Morphir_IR_Value.Value<Ta, Va>);
    }
    function decodeDestructure<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): Destructure<Va, Ta>;
    function encodeDestructure<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: Destructure<Va, Ta>): any;
    class Field<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Value<Ta, Va>;
        arg3: Morphir_IR_Name.Name;
        kind: "Field";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Value<Ta, Va>, arg3: Morphir_IR_Name.Name);
    }
    function decodeField<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): Field<Va, Ta>;
    function encodeField<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: Field<Va, Ta>): any;
    class FieldFunction<Va> {
        arg1: Va;
        arg2: Morphir_IR_Name.Name;
        kind: "FieldFunction";
        constructor(arg1: Va, arg2: Morphir_IR_Name.Name);
    }
    function decodeFieldFunction<Va>(decodeVa: (input: any) => Va, input: any): FieldFunction<Va>;
    function encodeFieldFunction<Va>(encodeVa: (value: Va) => any, value: FieldFunction<Va>): any;
    class IfThenElse<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Value<Ta, Va>;
        arg3: Morphir_IR_Value.Value<Ta, Va>;
        arg4: Morphir_IR_Value.Value<Ta, Va>;
        kind: "IfThenElse";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Value<Ta, Va>, arg3: Morphir_IR_Value.Value<Ta, Va>, arg4: Morphir_IR_Value.Value<Ta, Va>);
    }
    function decodeIfThenElse<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): IfThenElse<Va, Ta>;
    function encodeIfThenElse<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: IfThenElse<Va, Ta>): any;
    class Lambda<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Pattern<Va>;
        arg3: Morphir_IR_Value.Value<Ta, Va>;
        kind: "Lambda";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Pattern<Va>, arg3: Morphir_IR_Value.Value<Ta, Va>);
    }
    function decodeLambda<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): Lambda<Va, Ta>;
    function encodeLambda<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: Lambda<Va, Ta>): any;
    class LetDefinition<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Name.Name;
        arg3: Morphir_IR_Value.Definition<Ta, Va>;
        arg4: Morphir_IR_Value.Value<Ta, Va>;
        kind: "LetDefinition";
        constructor(arg1: Va, arg2: Morphir_IR_Name.Name, arg3: Morphir_IR_Value.Definition<Ta, Va>, arg4: Morphir_IR_Value.Value<Ta, Va>);
    }
    function decodeLetDefinition<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): LetDefinition<Va, Ta>;
    function encodeLetDefinition<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: LetDefinition<Va, Ta>): any;
    class LetRecursion<Va, Ta> {
        arg1: Va;
        arg2: Map<Morphir_IR_Name.Name, Morphir_IR_Value.Definition<Ta, Va>>;
        arg3: Morphir_IR_Value.Value<Ta, Va>;
        kind: "LetRecursion";
        constructor(arg1: Va, arg2: Map<Morphir_IR_Name.Name, Morphir_IR_Value.Definition<Ta, Va>>, arg3: Morphir_IR_Value.Value<Ta, Va>);
    }
    function decodeLetRecursion<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): LetRecursion<Va, Ta>;
    function encodeLetRecursion<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: LetRecursion<Va, Ta>): any;
    class List<Va, Ta> {
        arg1: Va;
        arg2: Array<Morphir_IR_Value.Value<Ta, Va>>;
        kind: "List";
        constructor(arg1: Va, arg2: Array<Morphir_IR_Value.Value<Ta, Va>>);
    }
    function decodeList<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): List<Va, Ta>;
    function encodeList<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: List<Va, Ta>): any;
    class Literal<Va> {
        arg1: Va;
        arg2: Morphir_IR_Literal.Literal;
        kind: "Literal";
        constructor(arg1: Va, arg2: Morphir_IR_Literal.Literal);
    }
    function decodeLiteral<Va>(decodeVa: (input: any) => Va, input: any): Literal<Va>;
    function encodeLiteral<Va>(encodeVa: (value: Va) => any, value: Literal<Va>): any;
    class PatternMatch<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Value<Ta, Va>;
        arg3: Array<[
            Morphir_IR_Value.Pattern<Va>,
            Morphir_IR_Value.Value<Ta, Va>
        ]>;
        kind: "PatternMatch";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Value<Ta, Va>, arg3: Array<[
            Morphir_IR_Value.Pattern<Va>,
            Morphir_IR_Value.Value<Ta, Va>
        ]>);
    }
    function decodePatternMatch<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): PatternMatch<Va, Ta>;
    function encodePatternMatch<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: PatternMatch<Va, Ta>): any;
    class Record<Va, Ta> {
        arg1: Va;
        arg2: Map<Morphir_IR_Name.Name, Morphir_IR_Value.Value<Ta, Va>>;
        kind: "Record";
        constructor(arg1: Va, arg2: Map<Morphir_IR_Name.Name, Morphir_IR_Value.Value<Ta, Va>>);
    }
    function decodeRecord<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): Record<Va, Ta>;
    function encodeRecord<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: Record<Va, Ta>): any;
    class Reference<Va> {
        arg1: Va;
        arg2: Morphir_IR_FQName.FQName;
        kind: "Reference";
        constructor(arg1: Va, arg2: Morphir_IR_FQName.FQName);
    }
    function decodeReference<Va>(decodeVa: (input: any) => Va, input: any): Reference<Va>;
    function encodeReference<Va>(encodeVa: (value: Va) => any, value: Reference<Va>): any;
    class Tuple<Va, Ta> {
        arg1: Va;
        arg2: Array<Morphir_IR_Value.Value<Ta, Va>>;
        kind: "Tuple";
        constructor(arg1: Va, arg2: Array<Morphir_IR_Value.Value<Ta, Va>>);
    }
    function decodeTuple<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): Tuple<Va, Ta>;
    function encodeTuple<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: Tuple<Va, Ta>): any;
    class Unit<Va> {
        arg1: Va;
        kind: "Unit";
        constructor(arg1: Va);
    }
    function decodeUnit<Va>(decodeVa: (input: any) => Va, input: any): Unit<Va>;
    function encodeUnit<Va>(encodeVa: (value: Va) => any, value: Unit<Va>): any;
    class UpdateRecord<Va, Ta> {
        arg1: Va;
        arg2: Morphir_IR_Value.Value<Ta, Va>;
        arg3: Map<Morphir_IR_Name.Name, Morphir_IR_Value.Value<Ta, Va>>;
        kind: "UpdateRecord";
        constructor(arg1: Va, arg2: Morphir_IR_Value.Value<Ta, Va>, arg3: Map<Morphir_IR_Name.Name, Morphir_IR_Value.Value<Ta, Va>>);
    }
    function decodeUpdateRecord<Va, Ta>(decodeVa: (input: any) => Va, decodeTa: (input: any) => Ta, input: any): UpdateRecord<Va, Ta>;
    function encodeUpdateRecord<Va, Ta>(encodeVa: (value: Va) => any, encodeTa: (value: Ta) => any, value: UpdateRecord<Va, Ta>): any;
    class Variable<Va> {
        arg1: Va;
        arg2: Morphir_IR_Name.Name;
        kind: "Variable";
        constructor(arg1: Va, arg2: Morphir_IR_Name.Name);
    }
    function decodeVariable<Va>(decodeVa: (input: any) => Va, input: any): Variable<Va>;
    function encodeVariable<Va>(encodeVa: (value: Va) => any, value: Variable<Va>): any;
}
